@@ -43,12 +43,24 @@ def mk_video_src(args, videocaps):
     # args come from command line
     # videocaps come from voctocore
 
-    if args.video_source == 'dv':
+    if args.video_source == 'test':
+
+        video_src = "videotestsrc name=videosrc {attribs}"
+
+    elif args.video_source == 'spacescope':
+        # Stereo visualizer
+        # pair up with audoi test beep for a handy AV sync test.
+        video_src = """
+      audio_tee. ! queue !
+    spacescope shader=none style=lines {attribs}
+           """
+
+    elif args.video_source == 'dv':
         video_src = """
             dv1394src name=videosrc {attribs} !
         dvdemux name=demux !
         queue max-size-time=4000000000 !
-        dvdec !
+        dvdec
             """
 
     elif args.video_source == 'hdv':
@@ -56,46 +68,47 @@ def mk_video_src(args, videocaps):
             hdv1394src {attribs} name=videosrc !
         tsdemux !
         queue max-size-time=4000000000 !
-        decodebin !
+        decodebin
             """
 
     elif args.video_source in ('hdmi2usb', 'uvc-mjpeg'):
         video_src = """
-            v4l2src {attribs} name=videosrc !
+            v4l2src {attribs} name=videosrc
         """
-        video_src += """
+        video_src += """ !
                 queue max-size-time=4000000000 !
         image/jpeg,{caps} !
-                jpegdec !
+                jpegdec
             """.format(caps=extract_caps(videocaps))
 
     elif args.video_source == 'uvc-raw':
 
         video_src = """
-            v4l2src {attribs} name=videosrc !
+            v4l2src {attribs} name=videosrc
         """
-        video_src += """
-        video/x-raw,{caps} !
+        video_src += """ !
+        video/x-raw,{caps}
         """.format(
             caps=extract_caps(videocaps))
 
     elif args.video_source == 'ximage':
+        # Doesn't work with Wayland.
         # startx=0 starty=0 endx=1919 endy=1079 !
         video_src = """
             ximagesrc {attribs} name=videosrc
-                   use-damage=false !
+                   use-damage=false
             """
 
     elif args.video_source == 'blackmagic':
         video_src = """
-            decklinkvideosrc name=videosrc {attribs} !
+            decklinkvideosrc name=videosrc {attribs}
             """
 
     elif args.video_source == 'png':
         video_src = """
             multifilesrc {attribs}
                 caps="image/png" !
-            pngdec !
+            pngdec
             """
         # tip: --video-attribs "loop=true"
         # tip: --video-attribs "location=sc1204a.png"
@@ -105,7 +118,7 @@ def mk_video_src(args, videocaps):
             multifilesrc {attribs} !
             decodebin name=src
             src. !
-            queue !
+            queue
             """
 
     elif args.video_source == 'rtmp':
@@ -113,45 +126,44 @@ def mk_video_src(args, videocaps):
             rtmpsrc {attribs} !
             decodebin name=src
             src. !
-            queue !
+            queue
             """
+    else:
+        # What I wish I had done from the start.
+        # similar to args.src, only just for the video part.
+        video_src = args.video_source
 
-    elif args.video_source == 'test':
+    if args.debug:
+        # text to overlay on top of video
 
-        video_src = """
-    videotestsrc name=videosrc {attribs} !
-    """
-        # things to render as text ontop of test video
-        video_src += """
-    clockoverlay
-        text="Source: {hostname}\nCaps: {videocaps}\nAttribs: {attribs}\n"
-        halignment=left line-alignment=left !
-            """.format(hostname=socket.gethostname(),
-                    videocaps=videocaps,
-                    attribs=args.video_attribs)
+        hostname=socket.gethostname()
+        text = f"Ingest Host: {hostname}\n"
+
+        text += f"--video-source: {args.video_source}\n"
+        if args.video_attribs:
+            text += f"video_attribs: {args.video_attribs}\n"
+
+        text += f"--audio-source: {args.audio_source}\n"
+        if args.audio_attribs:
+            text += f"audio_attribs: {args.audio_attribs}\n"
+
+        text += f"Server Caps: {videocaps}\n"
+
+        video_src += f' ! clockoverlay text="{text}" halignment=left line-alignment=left'
 
-    elif args.video_source == 'spacescope':
-        # Stereo visualizer
-        # pair up with test beep for a handy AV sync test.
-        video_src = """
-      audio_tee. ! queue !
-    spacescope shader=none style=lines {attribs} !
-           """
 
     if args.monitor:
 
-        videosink=args.monitor
-
-        video_src += f"""
+        video_src += f""" !
             tee name=t ! queue !
-                    videoconvert ! {videosink} sync=false
-                    t. ! queue !
+                    videoconvert ! {args.monitor} sync=false
+                    t. ! queue
             """
 
     if args.video_elements:
-        video_src += args.video_elements + " !\n"
+        video_src += f" ! {args.video_elements}\n"
 
-    video_src += videocaps + " !\n"
+    video_src += f" ! {videocaps}\n"
 
     video_src = video_src.format(attribs=args.video_attribs)
 
@@ -167,12 +179,17 @@ def mk_audio_src(args, audiocaps):
         'audiocaps': audiocaps,
     }
 
-    if args.audio_source in ['dv', 'hdv']:
+    if args.audio_source == 'test':
+        audio_src = """
+            audiotestsrc wave=ticks freq=330 {attribs} name=audiosrc
+            """
+
+    elif args.audio_source in ['dv', 'hdv']:
         # this only works if video is from DV also.
         # or some gst source that gets demux ed
         audio_src = """
             demux.audio !
-                queue !
+                queue
                 """
 
     elif args.audio_source in ('file', 'rtmp'):
@@ -180,38 +197,39 @@ def mk_audio_src(args, audiocaps):
         # some gst source that gets demux ed, I guess.
         audio_src = """
         src. !
-                queue !
+                queue
                 """
 
     elif args.audio_source == 'pulse':
         audio_src = """
                 pulsesrc {attribs} {base_audio_attribs} name=audiosrc !
-                queue max-size-time=4000000000 !
+                queue max-size-time=4000000000
                 """
 
     elif args.audio_source == 'alsa':
         audio_src = """
                 alsasrc {attribs} {base_audio_attribs} name=audiosrc !
-                queue max-size-time=4000000000 !
+                queue max-size-time=4000000000
                 """
 
     elif args.audio_source == 'blackmagic':
         audio_src = """
-            decklinkaudiosrc name=audiosrc {attribs} !
+            decklinkaudiosrc name=audiosrc {attribs}
             """
+    else:
+        # What I wish I had done from the start.
+        # similar to args.src, only just for the audio part.
+        audio_src = args.audio_source
 
-    elif args.audio_source == 'test':
-        audio_src = """
-            audiotestsrc wave=ticks freq=330 {attribs} name=audiosrc !
-            """
 
     if args.audio_elements:
-        audio_src += args.audio_elements + " !\n"
+        audio_src += f" ! {args.audio_elements}\n"
+
 
     audio_src += """
-       {audiocaps} !
-    tee name=audio_tee
-      audio_tee. ! queue !
+    !   {audiocaps}
+    ! tee name=audio_tee
+      audio_tee. ! queue
       """
     audio_src = audio_src.format(**d)
 
@@ -243,17 +261,17 @@ def mk_pipeline(args, server_caps, core_ip):
         audio_src = mk_audio_src(args, server_caps['audiocaps'])
         src =  """
             {video_src}
-             mux.
+             ! mux.
             {audio_src}
-             mux.
-            matroskamux name=mux !
+             ! mux.
+            matroskamux name=mux
         """.format(video_src=video_src, audio_src=audio_src)
 
     client = mk_client(core_ip, args.port)
 
     pipeline = """
             {src}
-    {client}
+    ! {client}
     """.format(src=src, client=client)
 
     # remove blank lines to make it more human readable
@@ -268,7 +286,7 @@ def mk_pipeline(args, server_caps, core_ip):
 
         # escape the ! because bash
         # asl2: ! is interpreted as a command history metacharacter
-        gst_cmd = gst_cmd.replace("!", " \! ")
+        gst_cmd = gst_cmd.replace("!", " \\! ")
 
         # remove all the \n to make it easy to cut/paste into shell
         gst_cmd = gst_cmd.replace("\n", " ")
@@ -364,18 +382,18 @@ def run_pipeline(pipeline, clock, audio_delay=0, video_delay=0):
     senderPipeline.bus.connect("message::eos", on_eos)
     senderPipeline.bus.connect("message::error", on_error)
 
-    print("playing...")
+    print("ingest.py: streaming to core...")
     senderPipeline.set_state(Gst.State.PLAYING)
 
     mainloop = GLib.MainLoop()
     try:
         mainloop.run()
     except KeyboardInterrupt:
-        print('Terminated via Ctrl-C')
+        print('ingest.py: Terminated via Ctrl-C')
 
-    print('Shutting down...')
+    print('ingest.py: Shutting down...')
     senderPipeline.set_state(Gst.State.NULL)
-    print('Done.')
+    print('ingest.py: Done.')
 
     return
 
@@ -401,9 +419,6 @@ def get_args():
 
     parser.add_argument(
         '--video-source', action='store',
-        choices=[
-            'dv', 'hdv', 'udp_h264', 'hdmi2usb', 'uvc-mjpeg', 'uvc-raw',
-            'blackmagic', 'ximage', 'png', 'file', 'rtmp', 'test', 'spacescope'],
         default='test',
         help="Where to get video from")
 
@@ -424,8 +439,6 @@ def get_args():
 
     parser.add_argument(
         '--audio-source', action='store',
-        choices=['dv', 'hdv', 'file',
-            'alsa', 'pulse', 'blackmagic', 'rtmp', 'test',],
         default='test',
         help="Where to get audio from")
 
